@@ -59,10 +59,8 @@ namespace Rand
 
         explicit constexpr PCG32( const uint64_t seed = defaultSeed, const uint64_t stream = defaultStream )
             : _state( 0 )
-            , _increment( makeOdd( stream ) )
+            , _increment( stream )
         {
-            // _increment must be odd
-            assert( _increment % 2 == 1 );
             advanceState();
             _state += seed;
             advanceState();
@@ -85,7 +83,7 @@ namespace Rand
 
         constexpr void setStream( const uint64_t stream )
         {
-            _increment = makeOdd( stream );
+            _increment = stream;
         }
 
     private:
@@ -98,11 +96,6 @@ namespace Rand
             return ( value >> rotations ) | ( value << ( ( ~( rotations - 1 ) ) & 31 ) );
         }
 
-        static constexpr uint64_t makeOdd( const uint64_t value )
-        {
-            return ( value << 1U ) | 1U;
-        }
-
         // Defines a new templated false value to be used in static_assert
         // See https://devblogs.microsoft.com/oldnewthing/20200311-00/?p=103553
         template <typename T>
@@ -253,4 +246,11 @@ namespace Rand
     private:
         int32_t Get( const std::function<uint32_t( uint32_t )> & randomFunc ) const;
     };
+
+    template <typename Seed, typename Value, std::enable_if_t<std::is_same_v<Seed, uint32_t> || std::is_same_v<Seed, uint64_t>, bool> = true>
+    void combineSeedWithValueHash( Seed & seed, const Value & v )
+    {
+        std::hash<Value> hasher;
+        seed ^= hasher( v ) + 0x9e3779b9 + ( seed << 6 ) + ( seed >> 2 );
+    }
 }
