@@ -1018,8 +1018,8 @@ namespace ICN
         BUTTONS_FILE_DIALOG_EVIL,
         BUTTONS_EDITOR_FILE_DIALOG_GOOD,
         BUTTONS_EDITOR_FILE_DIALOG_EVIL,
-        BUTTON_INFO_GOOD,
-        BUTTON_INFO_EVIL,
+        BUTTONS_ADVENTURE_OPTIONS_DIALOG_GOOD,
+        BUTTONS_ADVENTURE_OPTIONS_DIALOG_EVIL,
         BUTTON_QUIT_GOOD,
         BUTTON_QUIT_EVIL,
 
@@ -1103,7 +1103,15 @@ namespace ICN
         BUTTON_SELECT_GOOD,
         BUTTON_SELECT_EVIL,
 
+        BUTTON_TOGGLE_ALL_ON_GOOD,
+        BUTTON_TOGGLE_ALL_OFF_GOOD,
+        BUTTON_TOGGLE_ALL_ON_EVIL,
+        BUTTON_TOGGLE_ALL_OFF_EVIL,
+
+        EMPTY_OPTION_ICON_BACKGROUND,
         GAME_OPTION_ICON,
+        RESOLUTION_ICON,
+        ARMY_ESTIMATION_ICON,
 
         DIFFICULTY_ICON_EASY,
         DIFFICULTY_ICON_NORMAL,
