@@ -309,7 +309,7 @@ namespace
 
         LocalEvent & le = LocalEvent::Get();
         while ( le.HandleEvents() ) {
-            buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+            buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
             bool redrawScreen = false;
 
@@ -511,8 +511,8 @@ void Battle::GetSummaryParams( const uint32_t res1, const uint32_t res2, const H
 // Returns true if player wants to restart the battle
 bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<Artifact> & artifacts, const bool allowToRestart ) const
 {
-    const bool attackerIsHuman = _army1->GetControl() & CONTROL_HUMAN;
-    const bool defenderIsHuman = _army2->GetControl() & CONTROL_HUMAN;
+    const bool attackerIsHuman = _attackingArmy->GetControl() & CONTROL_HUMAN;
+    const bool defenderIsHuman = _defendingArmy->GetControl() & CONTROL_HUMAN;
 
     if ( !attackerIsHuman && !defenderIsHuman ) {
         // AI vs AI battle, this dialog should not be shown
@@ -543,22 +543,24 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
     LoopedAnimationSequence sequence;
 
     fheroes2::FontType summaryTitleFont = fheroes2::FontType::normalWhite();
-    if ( ( res.army1 & RESULT_WINS ) && attackerIsHuman ) {
-        GetSummaryParams( res.army1, res.army2, _army1->GetCommander(), res.exp1, _army2->GetSurrenderCost(), sequence, title, surrenderText, outcomeText );
+    if ( ( res.attacker & RESULT_WINS ) && attackerIsHuman ) {
+        GetSummaryParams( res.attacker, res.defender, _attackingArmy->GetCommander(), res.attackerExperience, _defendingArmy->GetSurrenderCost(), sequence, title,
+                          surrenderText, outcomeText );
         summaryTitleFont = fheroes2::FontType::normalYellow();
         AudioManager::PlayMusic( MUS::BATTLEWIN, Music::PlaybackMode::PLAY_ONCE );
     }
-    else if ( ( res.army2 & RESULT_WINS ) && defenderIsHuman ) {
-        GetSummaryParams( res.army2, res.army1, _army2->GetCommander(), res.exp2, _army1->GetSurrenderCost(), sequence, title, surrenderText, outcomeText );
+    else if ( ( res.defender & RESULT_WINS ) && defenderIsHuman ) {
+        GetSummaryParams( res.defender, res.attacker, _defendingArmy->GetCommander(), res.defenderExperience, _attackingArmy->GetSurrenderCost(), sequence, title,
+                          surrenderText, outcomeText );
         summaryTitleFont = fheroes2::FontType::normalYellow();
         AudioManager::PlayMusic( MUS::BATTLEWIN, Music::PlaybackMode::PLAY_ONCE );
     }
     else if ( attackerIsHuman ) {
-        GetSummaryParams( res.army1, res.army2, _army1->GetCommander(), res.exp1, 0, sequence, title, surrenderText, outcomeText );
+        GetSummaryParams( res.attacker, res.defender, _attackingArmy->GetCommander(), res.attackerExperience, 0, sequence, title, surrenderText, outcomeText );
         AudioManager::PlayMusic( MUS::BATTLELOSE, Music::PlaybackMode::PLAY_ONCE );
     }
     else if ( defenderIsHuman ) {
-        GetSummaryParams( res.army2, res.army1, _army2->GetCommander(), res.exp2, 0, sequence, title, surrenderText, outcomeText );
+        GetSummaryParams( res.defender, res.attacker, _defendingArmy->GetCommander(), res.defenderExperience, 0, sequence, title, surrenderText, outcomeText );
         AudioManager::PlayMusic( MUS::BATTLELOSE, Music::PlaybackMode::PLAY_ONCE );
     }
 
@@ -620,11 +622,11 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
     text.set( _( "Attacker" ), casualtiesFont );
     text.draw( summaryRoi.x + ( summaryRoi.width - text.width() ) / 2, casualtiesOffsetY + 15, display );
 
-    const Troops killed1 = _army1->GetKilledTroops();
-    const Troops killed2 = _army2->GetKilledTroops();
+    const Troops killedInAttackingArmy = _attackingArmy->GetKilledTroops();
+    const Troops killedInDefendingArmy = _defendingArmy->GetKilledTroops();
 
-    if ( killed1.isValid() ) {
-        Army::drawSingleDetailedMonsterLine( killed1, summaryRoi.x + 13, casualtiesOffsetY + 36, roi.width - 47 );
+    if ( killedInAttackingArmy.isValid() ) {
+        Army::drawSingleDetailedMonsterLine( killedInAttackingArmy, summaryRoi.x + 13, casualtiesOffsetY + 36, roi.width - 47 );
     }
     else {
         text.set( _( "None" ), casualtiesFont );
@@ -635,8 +637,8 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
     text.set( _( "Defender" ), casualtiesFont );
     text.draw( summaryRoi.x + ( summaryRoi.width - text.width() ) / 2, casualtiesOffsetY + 75, display );
 
-    if ( killed2.isValid() ) {
-        Army::drawSingleDetailedMonsterLine( killed2, summaryRoi.x + 13, casualtiesOffsetY + 96, roi.width - 47 );
+    if ( killedInDefendingArmy.isValid() ) {
+        Army::drawSingleDetailedMonsterLine( killedInDefendingArmy, summaryRoi.x + 13, casualtiesOffsetY + 96, roi.width - 47 );
     }
     else {
         text.set( _( "None" ), casualtiesFont );
@@ -672,10 +674,10 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
     int sequenceId = sequence.id();
 
     while ( le.HandleEvents() ) {
-        buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+        buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
         if ( allowToRestart ) {
-            buttonRestart->drawOnState( le.isMouseLeftButtonPressedInArea( buttonRestart->area() ) );
+            buttonRestart->drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonRestart->area() ) );
         }
 
         if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOk.area() ) ) {
@@ -704,8 +706,10 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
     }
 
     if ( !artifacts.empty() ) {
-        const HeroBase * winner = ( res.army1 & RESULT_WINS ? _army1->GetCommander() : ( res.army2 & RESULT_WINS ? _army2->GetCommander() : nullptr ) );
-        const HeroBase * loser = ( res.army1 & RESULT_LOSS ? _army1->GetCommander() : ( res.army2 & RESULT_LOSS ? _army2->GetCommander() : nullptr ) );
+        const HeroBase * winner
+            = ( res.attacker & RESULT_WINS ? _attackingArmy->GetCommander() : ( res.defender & RESULT_WINS ? _defendingArmy->GetCommander() : nullptr ) );
+        const HeroBase * loser
+            = ( res.attacker & RESULT_LOSS ? _attackingArmy->GetCommander() : ( res.defender & RESULT_LOSS ? _defendingArmy->GetCommander() : nullptr ) );
 
         // Cannot transfer artifacts
         if ( winner == nullptr || loser == nullptr ) {
@@ -796,7 +800,7 @@ bool Battle::Arena::DialogBattleSummary( const Result & res, const std::vector<A
             }
 
             while ( le.HandleEvents() ) {
-                buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+                buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
                 // Display captured artifact info on right click
                 if ( le.isMouseRightButtonPressedInArea( artifactArea ) ) {
@@ -876,7 +880,7 @@ void Battle::Arena::DialogBattleNecromancy( const uint32_t raiseCount )
     int sequenceId = sequence.id();
 
     while ( le.HandleEvents() ) {
-        buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+        buttonOk.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonOk.area() ) );
 
         if ( Game::HotKeyCloseWindow() || le.MouseClickLeft( buttonOk.area() ) ) {
             break;
@@ -893,7 +897,7 @@ int Battle::Arena::DialogBattleHero( HeroBase & hero, const bool buttons, Status
     Cursor & cursor = Cursor::Get();
     cursor.SetThemes( Cursor::POINTER );
 
-    const int currentColor = GetCurrentColor();
+    const PlayerColor currentColor = GetCurrentColor();
     const bool readonly = ( currentColor != hero.GetColor() || !buttons );
     const fheroes2::Sprite & dialog = fheroes2::AGG::GetICN( ( conf.isEvilInterfaceEnabled() ? ICN::VGENBKGE : ICN::VGENBKG ), 0 );
 
@@ -914,7 +918,7 @@ int Battle::Arena::DialogBattleHero( HeroBase & hero, const bool buttons, Status
     const fheroes2::Rect portraitArea( pos_rt.x + 7, pos_rt.y + 35, 113, 108 );
 
     hero.PortraitRedraw( pos_rt.x + 12, pos_rt.y + 42, PORT_BIG, display );
-    int col = ( Color::NONE == hero.GetColor() ? 1 : Color::GetIndex( hero.GetColor() ) + 1 );
+    const int col = ( PlayerColor::NONE == hero.GetColor() ? 1 : Color::GetIndex( hero.GetColor() ) + 1 );
     fheroes2::Blit( fheroes2::AGG::GetICN( ICN::VIEWGEN, col ), display, pos_rt.x + 133, pos_rt.y + 36 );
 
     std::string str = hero.isCaptain() ? _( "Captain of %{name}" ) : _( "%{name} the %{race}" );
@@ -1018,20 +1022,20 @@ int Battle::Arena::DialogBattleHero( HeroBase & hero, const bool buttons, Status
     // The Hero Screen is available for a Hero only (not Captain) and only when the corresponding player has a turn.
     Heroes * heroForHeroScreen = ( currentColor == hero.GetColor() ) ? dynamic_cast<Heroes *>( &hero ) : nullptr;
 
-    std::string statusMessage = _( "Hero's Options" );
+    std::string statusMessage;
 
     while ( le.HandleEvents() && !result ) {
         if ( btnCast.isEnabled() ) {
-            btnCast.drawOnState( le.isMouseLeftButtonPressedInArea( btnCast.area() ) );
+            btnCast.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnCast.area() ) );
         }
         if ( btnRetreat.isEnabled() ) {
-            btnRetreat.drawOnState( le.isMouseLeftButtonPressedInArea( btnRetreat.area() ) );
+            btnRetreat.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnRetreat.area() ) );
         }
         if ( btnSurrender.isEnabled() ) {
-            btnSurrender.drawOnState( le.isMouseLeftButtonPressedInArea( btnSurrender.area() ) );
+            btnSurrender.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnSurrender.area() ) );
         }
 
-        btnClose.drawOnState( le.isMouseLeftButtonPressedInArea( btnClose.area() ) );
+        btnClose.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnClose.area() ) );
 
         // The Cast Spell is available for a hero and a captain.
         if ( le.isMouseCursorPosInArea( btnCast.area() ) && currentColor == hero.GetColor() ) {
@@ -1109,7 +1113,7 @@ int Battle::Arena::DialogBattleHero( HeroBase & hero, const bool buttons, Status
         }
 
         if ( statusMessage != status.getMessage() ) {
-            status.setMessage( statusMessage, false );
+            status.setMessage( std::move( statusMessage ), false );
             status.redraw( display );
             display.render( status );
         }
@@ -1214,13 +1218,13 @@ bool Battle::DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdo
 
     while ( le.HandleEvents() && !result ) {
         if ( btnAccept.isEnabled() ) {
-            btnAccept.drawOnState( le.isMouseLeftButtonPressedInArea( btnAccept.area() ) );
+            btnAccept.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnAccept.area() ) );
         }
 
-        btnDecline.drawOnState( le.isMouseLeftButtonPressedInArea( btnDecline.area() ) );
+        btnDecline.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnDecline.area() ) );
 
         if ( btnMarket.isEnabled() ) {
-            btnMarket.drawOnState( le.isMouseLeftButtonPressedInArea( btnMarket.area() ) );
+            btnMarket.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( btnMarket.area() ) );
         }
 
         if ( btnAccept.isEnabled() && ( Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) || le.MouseClickLeft( btnAccept.area() ) ) ) {
