@@ -140,8 +140,10 @@ public:
         Close, // Close the dialog.
         NextCastle, // Open main dialog of the next castle.
         PreviousCastle, // Open main dialog of the previous castle.
-        NextCostructionWindow, // Open construction dialog of the next castle.
-        PreviousCostructionWindow // Open construction dialog of the previous castle.
+        NextConstructionWindow, // Open construction dialog of the next castle.
+        PreviousConstructionWindow, // Open construction dialog of the previous castle.
+        NextMageGuildWindow, // Open Mage Guild dialog of the next castle.
+        PreviousMageGuildWindow, // Open Mage Guild dialog of the previous castle.
     };
 
     Castle() = default;
@@ -222,14 +224,14 @@ public:
         return _race == Race::WZRD;
     }
 
-    bool isLibraryBuild() const
+    bool isLibraryBuilt() const
     {
         return _race == Race::WZRD && isBuild( BUILD_SPEC );
     }
 
     void trainHeroInMageGuild( HeroBase & hero ) const
     {
-        _mageGuild.trainHero( hero, GetLevelMageGuild(), isLibraryBuild() );
+        _mageGuild.trainHero( hero, GetLevelMageGuild(), isLibraryBuilt() );
     }
 
     bool isFortificationBuilt() const
@@ -287,7 +289,7 @@ public:
 
     void DrawImageCastle( const fheroes2::Point & pt ) const;
 
-    CastleDialogReturnValue OpenDialog( const bool openConstructionWindow, const bool fade, const bool renderBackgroundDialog );
+    CastleDialogReturnValue OpenDialog( const bool openConstructionWindow, const bool openMageGuildWindow, const bool fade, const bool renderBackgroundDialog );
 
     int GetAttackModificator( const std::string * /* unused */ ) const
     {
@@ -389,6 +391,13 @@ private:
         RecruitHero // Recruit a hero.
     };
 
+    enum class MageGuildDialogResult : uint8_t
+    {
+        DoNothing,
+        NextMageGuildWindow, // Open Mage Guild dialog for the next castle.
+        PrevMageGuildWindow, // Open Mage Guild dialog for the previous castle.
+    };
+
     // Checks whether this particular building is currently built in the castle (unlike
     // the isBuild(), upgraded versions of the same building are not taken into account)
     bool _isExactBuildingBuilt( const uint32_t buildingToCheck ) const;
@@ -400,7 +409,7 @@ private:
 
     void _openTavern() const;
     void _openWell();
-    void _openMageGuild( const Heroes * hero ) const;
+    MageGuildDialogResult _openMageGuild( const Heroes * hero ) const;
     void _joinRNDArmy();
     void _postLoad();
 
