@@ -30,8 +30,11 @@
 #include "history_manager.h"
 #include "interface_base.h"
 #include "map_format_info.h"
+#include "map_random_generator.h"
 #include "timing.h"
 
+enum class PlayerColor : uint8_t;
+
 namespace Maps
 {
     class Tile;
@@ -52,7 +55,7 @@ namespace Interface
         void reset() override;
 
         // Start Map Editor interface main function.
-        fheroes2::GameMode startEdit( const bool isNewMap );
+        fheroes2::GameMode startEdit();
 
         static fheroes2::GameMode eventLoadMap();
         static fheroes2::GameMode eventNewMap();
@@ -93,12 +96,19 @@ namespace Interface
             _cursorUpdater = cursorUpdater;
         }
 
+        // Generate a random map and start Map Editor interface main function.
+        bool generateRandomMap( const int32_t mapWidth );
+
+        bool generateNewMap( const int32_t mapWidth );
+
         bool loadMap( const std::string & filePath );
 
         void saveMapToFile();
 
         void openMapSpecificationsDialog();
 
+        bool updateRandomMapConfiguration();
+
     private:
         class WarningMessage
         {
@@ -161,11 +171,15 @@ namespace Interface
 
         void _updateObjectUID( const uint32_t oldObjectUID, const uint32_t newObjectUID );
 
+        bool _placeCastle( const int32_t posX, const int32_t posY, const PlayerColor color, const int32_t type );
+
         EditorPanel _editorPanel;
 
         int32_t _areaSelectionStartTileId{ -1 };
         int32_t _tileUnderCursor{ -1 };
 
+        Maps::Random_Generator::Configuration _randomMapConfig;
+
         std::function<void( const int32_t )> _cursorUpdater;
 
         fheroes2::HistoryManager _historyManager;
