@@ -797,8 +797,8 @@ namespace
         if ( !completedScenario.getEndScenarioVideoPlayback().empty() ) {
             AudioManager::ResetAudio();
 
-            for ( const Campaign::ScenarioIntroVideoInfo & videoInfo : completedScenario.getEndScenarioVideoPlayback() ) {
-                Video::ShowVideo( videoInfo.fileName, videoInfo.action );
+            for ( const auto & item : completedScenario.getEndScenarioVideoPlayback() ) {
+                Video::ShowVideo( item );
             }
 
             AudioManager::ResetAudio();
@@ -820,8 +820,8 @@ namespace
         if ( !scenario.getStartScenarioVideoPlayback().empty() ) {
             AudioManager::ResetAudio();
 
-            for ( const Campaign::ScenarioIntroVideoInfo & videoInfo : scenario.getStartScenarioVideoPlayback() ) {
-                Video::ShowVideo( videoInfo.fileName, videoInfo.action );
+            for ( const auto & item : scenario.getStartScenarioVideoPlayback() ) {
+                Video::ShowVideo( item );
             }
 
             AudioManager::ResetAudio();
@@ -1261,7 +1261,7 @@ fheroes2::GameMode Game::CompleteCampaignScenario( const bool isLoadingSaveFile
         Video::Subtitle ratingSubtitle( ratingText, 5000, UINT32_MAX, { 475, 110 }, 140 );
 
         AudioManager::ResetAudio();
-        Video::ShowVideo( "WIN.SMK", Video::VideoAction::WAIT_FOR_USER_INPUT, { std::move( ratingSubtitle ) }, true );
+        Video::ShowVideo( { { "WIN.SMK", Video::VideoControl::PLAY_CUTSCENE_WAIT } }, { std::move( ratingSubtitle ) }, true );
 
         // fheroes2::PlayMusic is run here in order to start playing before displaying the high score.
         AudioManager::PlayMusicAsync( MUS::VICTORY, Music::PlaybackMode::REWIND_AND_PLAY_INFINITE );
