@@ -25,6 +25,7 @@
 
 #include <algorithm>
 
+#include "game_hotkeys.h"
 #include "localevent.h"
 #include "ui_button.h"
 #include "ui_scrollbar.h"
@@ -74,6 +75,11 @@ namespace Interface
         virtual void ActionListSingleClick( Item & ) = 0;
         virtual void ActionListPressRight( Item & ) = 0;
 
+        virtual void ActionListLongPress( Item & /*unused*/ )
+        {
+            // Do nothing.
+        }
+
         virtual void ActionListDoubleClick( Item & item, const fheroes2::Point & /*mousePos*/, int32_t /*itemOffsetX*/, int32_t /*itemOffsetY*/ )
         {
             ActionListDoubleClick( item );
@@ -121,6 +127,15 @@ namespace Interface
             _scrollbar.setImage( image );
         }
 
+        void updateScrollBarImage()
+        {
+            const int32_t scrollBarWidth = _scrollbar.width();
+
+            setScrollBarImage( fheroes2::generateScrollbarSlider( _scrollbar, false, _scrollbar.getArea().height, VisibleItemCount(), _size(),
+                                                                  { 0, 0, scrollBarWidth, 8 }, { 0, 7, scrollBarWidth, 8 } ) );
+            _scrollbar.moveToIndex( _topId );
+        }
+
         fheroes2::Scrollbar & GetScrollbar()
         {
             return _scrollbar;
@@ -205,6 +220,11 @@ namespace Interface
             return ( *content )[_currentId];
         }
 
+        int getCurrentId() const
+        {
+            return _currentId;
+        }
+
         Item * GetFromPosition( const fheroes2::Point & mp )
         {
             Verify();
@@ -333,8 +353,8 @@ namespace Interface
         {
             LocalEvent & le = LocalEvent::Get();
 
-            _buttonPgUp.drawOnState( le.isMouseLeftButtonPressedInArea( _buttonPgUp.area() ) );
-            _buttonPgDn.drawOnState( le.isMouseLeftButtonPressedInArea( _buttonPgDn.area() ) );
+            _buttonPgUp.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _buttonPgUp.area() ) );
+            _buttonPgDn.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( _buttonPgDn.area() ) );
 
             if ( !IsValid() ) {
                 return false;
@@ -448,6 +468,15 @@ namespace Interface
 
                 return true;
             }
+            if ( !_lockClick && le.MouseLongPressLeft( rtAreaItems ) ) {
+                const fheroes2::Point & mousePos = le.getMouseCursorPos();
+                const int id = ( mousePos.y - rtAreaItems.y ) * maxItems / rtAreaItems.height + _topId;
+                if ( static_cast<size_t>( id + 1 ) <= content->size() ) {
+                    Item & item = ( *content )[static_cast<size_t>( id )]; // id is always >= 0
+                    ActionListLongPress( item );
+                }
+                return true;
+            }
             if ( le.isMouseLeftButtonPressedInArea( _scrollbar.getArea() ) || le.isMouseLeftButtonPressedInArea( rtAreaItems ) ) {
                 const fheroes2::Point mousePosition = le.getMouseCursorPos();
 
@@ -530,6 +559,10 @@ namespace Interface
                         }
                         return true;
                     }
+                    if ( le.isMouseLeftButtonPressedInArea( rtAreaItems ) ) {
+                        ActionListLongPress( item );
+                        return true;
+                    }
 
                     if ( le.isMouseRightButtonPressedInArea( rtAreaItems ) ) {
                         ActionListPressRight( item, mousePos, rtAreaItems.x, rtAreaItems.y + offsetY );
