@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2023 - 2024                                             *
+ *   Copyright (C) 2023 - 2025                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,13 +21,24 @@
 #include "map_format_info.h"
 
 #include <array>
+#include <cassert>
 #include <cstddef>
+#include <initializer_list>
+#include <utility>
 
+#include "artifact.h"
+#include "mp2.h"
 #include "serialize.h"
 #include "zzlib.h"
 
 namespace Maps::Map_Format
 {
+    // This structure is deprecated. We are keeping it for backward compatibility.
+    struct StandardObjectMetadata
+    {
+        std::array<int32_t, 3> metadata{ 0 };
+    };
+
     // The following operators are used only inside this module, but they cannot be declared in an anonymous namespace due to the way ADL works
 
     OStreamBase & operator<<( OStreamBase & stream, const TileObjectInfo & object );
@@ -39,7 +50,6 @@ namespace Maps::Map_Format
     OStreamBase & operator<<( OStreamBase & stream, const DailyEvent & eventInfo );
     IStreamBase & operator>>( IStreamBase & stream, DailyEvent & eventInfo );
 
-    OStreamBase & operator<<( OStreamBase & stream, const StandardObjectMetadata & metadata );
     IStreamBase & operator>>( IStreamBase & stream, StandardObjectMetadata & metadata );
 
     OStreamBase & operator<<( OStreamBase & stream, const CastleMetadata & metadata );
@@ -59,6 +69,18 @@ namespace Maps::Map_Format
 
     OStreamBase & operator<<( OStreamBase & stream, const SelectionObjectMetadata & metadata );
     IStreamBase & operator>>( IStreamBase & stream, SelectionObjectMetadata & metadata );
+
+    OStreamBase & operator<<( OStreamBase & stream, const CapturableObjectMetadata & metadata );
+    IStreamBase & operator>>( IStreamBase & stream, CapturableObjectMetadata & metadata );
+
+    OStreamBase & operator<<( OStreamBase & stream, const MonsterMetadata & metadata );
+    IStreamBase & operator>>( IStreamBase & stream, MonsterMetadata & metadata );
+
+    OStreamBase & operator<<( OStreamBase & stream, const ArtifactMetadata & metadata );
+    IStreamBase & operator>>( IStreamBase & stream, ArtifactMetadata & metadata );
+
+    OStreamBase & operator<<( OStreamBase & stream, const ResourceMetadata & metadata );
+    IStreamBase & operator>>( IStreamBase & stream, ResourceMetadata & metadata );
 }
 
 namespace
@@ -72,7 +94,7 @@ namespace
     constexpr uint16_t minimumSupportedVersion{ 2 };
 
     // Change the version when there is a need to expand map format functionality.
-    constexpr uint16_t currentSupportedVersion{ 8 };
+    constexpr uint16_t currentSupportedVersion{ 10 };
 
     void convertFromV2ToV3( Maps::Map_Format::MapFormat & map )
     {
@@ -214,11 +236,64 @@ namespace
         }
     }
 
+    void convertFromV9ToV10( Maps::Map_Format::MapFormat & map, std::map<uint32_t, Maps::Map_Format::StandardObjectMetadata> standardMetadata )
+    {
+        static_assert( minimumSupportedVersion <= 9, "Remove this function." );
+
+        if ( map.version > 9 ) {
+            return;
+        }
+
+        map.monsterMetadata = {};
+        map.artifactMetadata = {};
+        map.resourceMetadata = {};
+
+        const auto & artifactObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_ARTIFACTS );
+        const auto & treasuresObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_TREASURES );
+
+        // In version 10 we updated metadata for monsters and artifacts. We have to move the original metadata for them from one structure to another.
+        for ( size_t tileId = 0; tileId < map.tiles.size(); ++tileId ) {
+            const auto & tile = map.tiles[tileId];
+
+            for ( const auto & object : tile.objects ) {
+                if ( object.group == Maps::ObjectGroup::MONSTERS ) {
+                    assert( standardMetadata.find( object.id ) != standardMetadata.end() );
+
+                    map.monsterMetadata[object.id].count = standardMetadata[object.id].metadata[0];
+                }
+                else if ( object.group == Maps::ObjectGroup::ADVENTURE_ARTIFACTS ) {
+                    const auto & objectInfo = artifactObjects[object.index];
+
+                    assert( standardMetadata.find( object.id ) != standardMetadata.end() );
+
+                    if ( objectInfo.objectType == MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT ) {
+                        map.artifactMetadata[object.id].radius = standardMetadata[object.id].metadata[0];
+                    }
+                    else if ( objectInfo.objectType == MP2::OBJ_ARTIFACT && objectInfo.metadata[0] == Artifact::SPELL_SCROLL ) {
+                        map.artifactMetadata[object.id].selected.push_back( standardMetadata[object.id].metadata[0] );
+                    }
+                    else {
+                        map.artifactMetadata[object.id] = {};
+                    }
+                }
+                else if ( object.group == Maps::ObjectGroup::ADVENTURE_TREASURES ) {
+                    assert( object.index < treasuresObjects.size() );
+                    const auto & objectInfo = treasuresObjects[object.index];
+
+                    if ( objectInfo.objectType == MP2::OBJ_RESOURCE ) {
+                        map.resourceMetadata[object.id].count = standardMetadata[object.id].metadata[0];
+                    }
+                }
+            }
+        }
+    }
+
     bool saveToStream( OStreamBase & stream, const Maps::Map_Format::BaseMapFormat & map )
     {
         stream << currentSupportedVersion << map.isCampaign << map.difficulty << map.availablePlayerColors << map.humanPlayerColors << map.computerPlayerColors
                << map.alliances << map.playerRace << map.victoryConditionType << map.isVictoryConditionApplicableForAI << map.allowNormalVictory
-               << map.victoryConditionMetadata << map.lossConditionType << map.lossConditionMetadata << map.size << map.mainLanguage << map.name << map.description;
+               << map.victoryConditionMetadata << map.lossConditionType << map.lossConditionMetadata << map.width << map.mainLanguage << map.name << map.description
+               << map.creatorNotes;
 
         return !stream.fail();
     }
@@ -232,15 +307,22 @@ namespace
 
         stream >> map.isCampaign >> map.difficulty >> map.availablePlayerColors >> map.humanPlayerColors >> map.computerPlayerColors >> map.alliances >> map.playerRace
             >> map.victoryConditionType >> map.isVictoryConditionApplicableForAI >> map.allowNormalVictory >> map.victoryConditionMetadata >> map.lossConditionType
-            >> map.lossConditionMetadata >> map.size;
+            >> map.lossConditionMetadata >> map.width;
 
-        if ( map.size <= 0 ) {
+        if ( map.width <= 0 ) {
             // This is not a correct map size.
             return false;
         }
 
         stream >> map.mainLanguage >> map.name >> map.description;
 
+        if ( map.version < 9 ) {
+            map.creatorNotes = {};
+        }
+        else {
+            stream >> map.creatorNotes;
+        }
+
         return !stream.fail();
     }
 
@@ -255,8 +337,9 @@ namespace
         RWStreamBuf compressed;
         compressed.setBigendian( true );
 
-        compressed << map.additionalInfo << map.tiles << map.dailyEvents << map.rumors << map.standardMetadata << map.castleMetadata << map.heroMetadata
-                   << map.sphinxMetadata << map.signMetadata << map.adventureMapEventMetadata << map.selectionObjectMetadata;
+        compressed << map.additionalInfo << map.tiles << map.dailyEvents << map.rumors << map.castleMetadata << map.heroMetadata << map.sphinxMetadata << map.signMetadata
+                   << map.adventureMapEventMetadata << map.selectionObjectMetadata << map.capturableObjectsMetadata << map.monsterMetadata << map.artifactMetadata
+                   << map.resourceMetadata;
 
         const std::vector<uint8_t> temp = Compression::zipData( compressed.data(), compressed.size() );
 
@@ -299,13 +382,24 @@ namespace
 
         decompressed >> map.additionalInfo >> map.tiles;
 
-        if ( map.tiles.size() != static_cast<size_t>( map.size ) * map.size ) {
+        if ( map.tiles.size() != static_cast<size_t>( map.width ) * map.width ) {
             map = {};
             return false;
         }
 
-        decompressed >> map.dailyEvents >> map.rumors >> map.standardMetadata >> map.castleMetadata >> map.heroMetadata >> map.sphinxMetadata >> map.signMetadata
-            >> map.adventureMapEventMetadata >> map.selectionObjectMetadata;
+        decompressed >> map.dailyEvents >> map.rumors;
+
+        std::map<uint32_t, Maps::Map_Format::StandardObjectMetadata> standardMetadata;
+        if ( map.version < 10 ) {
+            decompressed >> standardMetadata;
+        }
+
+        decompressed >> map.castleMetadata >> map.heroMetadata >> map.sphinxMetadata >> map.signMetadata >> map.adventureMapEventMetadata >> map.selectionObjectMetadata;
+
+        static_assert( minimumSupportedVersion <= 8, "Remove this check." );
+        if ( map.version > 8 ) {
+            decompressed >> map.capturableObjectsMetadata;
+        }
 
         convertFromV2ToV3( map );
         convertFromV3ToV4( map );
@@ -314,6 +408,13 @@ namespace
         convertFromV6ToV7( map );
         convertFromV7ToV8( map );
 
+        if ( map.version > 9 ) {
+            decompressed >> map.monsterMetadata >> map.artifactMetadata >> map.resourceMetadata;
+        }
+        else {
+            convertFromV9ToV10( map, std::move( standardMetadata ) );
+        }
+
         return !stream.fail();
     }
 }
@@ -332,12 +433,12 @@ namespace Maps::Map_Format
 
     OStreamBase & operator<<( OStreamBase & stream, const TileInfo & tile )
     {
-        return stream << tile.terrainIndex << tile.terrainFlag << tile.objects;
+        return stream << tile.terrainIndex << tile.terrainFlags << tile.objects;
     }
 
     IStreamBase & operator>>( IStreamBase & stream, TileInfo & tile )
     {
-        return stream >> tile.terrainIndex >> tile.terrainFlag >> tile.objects;
+        return stream >> tile.terrainIndex >> tile.terrainFlags >> tile.objects;
     }
 
     OStreamBase & operator<<( OStreamBase & stream, const DailyEvent & eventInfo )
@@ -352,11 +453,6 @@ namespace Maps::Map_Format
                >> eventInfo.repeatPeriodInDays >> eventInfo.resources;
     }
 
-    OStreamBase & operator<<( OStreamBase & stream, const StandardObjectMetadata & metadata )
-    {
-        return stream << metadata.metadata;
-    }
-
     IStreamBase & operator>>( IStreamBase & stream, StandardObjectMetadata & metadata )
     {
         return stream >> metadata.metadata;
@@ -434,6 +530,46 @@ namespace Maps::Map_Format
         return stream >> metadata.selectedItems;
     }
 
+    OStreamBase & operator<<( OStreamBase & stream, const CapturableObjectMetadata & metadata )
+    {
+        return stream << metadata.ownerColor;
+    }
+
+    IStreamBase & operator>>( IStreamBase & stream, CapturableObjectMetadata & metadata )
+    {
+        return stream >> metadata.ownerColor;
+    }
+
+    OStreamBase & operator<<( OStreamBase & stream, const MonsterMetadata & metadata )
+    {
+        return stream << metadata.count << metadata.joinCondition << metadata.isWeeklyGrowthDisabled << metadata.selected;
+    }
+
+    IStreamBase & operator>>( IStreamBase & stream, MonsterMetadata & metadata )
+    {
+        return stream >> metadata.count >> metadata.joinCondition >> metadata.isWeeklyGrowthDisabled >> metadata.selected;
+    }
+
+    OStreamBase & operator<<( OStreamBase & stream, const ArtifactMetadata & metadata )
+    {
+        return stream << metadata.radius << metadata.captureCondition << metadata.selected;
+    }
+
+    IStreamBase & operator>>( IStreamBase & stream, ArtifactMetadata & metadata )
+    {
+        return stream >> metadata.radius >> metadata.captureCondition >> metadata.selected;
+    }
+
+    OStreamBase & operator<<( OStreamBase & stream, const ResourceMetadata & metadata )
+    {
+        return stream << metadata.count;
+    }
+
+    IStreamBase & operator>>( IStreamBase & stream, ResourceMetadata & metadata )
+    {
+        return stream >> metadata.count;
+    }
+
     bool loadBaseMap( const std::string & path, BaseMapFormat & map )
     {
         if ( path.empty() ) {
