@@ -38,6 +38,8 @@
 class IStreamBase;
 class OStreamBase;
 
+enum class PlayerColor : uint8_t;
+
 inline constexpr int defaultBattleSpeed{ 4 };
 
 enum AdventureMapScrollSpeed : int
@@ -71,6 +73,12 @@ enum class InterfaceType : uint8_t
     DYNAMIC = 2
 };
 
+enum class SaveFileSortingMethod : uint8_t
+{
+    FILENAME,
+    TIMESTAMP,
+};
+
 class Settings
 {
 public:
@@ -196,6 +204,7 @@ public:
     bool isAutoSaveAtBeginningOfTurnEnabled() const;
     bool isBattleShowDamageInfoEnabled() const;
     bool isHideInterfaceEnabled() const;
+    bool isArmyEstimationViewNumeric() const;
     bool isEvilInterfaceEnabled() const;
 
     void setInterfaceType( InterfaceType type )
@@ -235,6 +244,16 @@ public:
         return _resolutionInfo;
     }
 
+    fheroes2::Point getSavedWindowPos() const
+    {
+        return _windowPos;
+    }
+
+    void setStartWindowPos( const fheroes2::Point pos )
+    {
+        _windowPos = pos;
+    }
+
     void EnablePriceOfLoyaltySupport( const bool set );
 
     void SetGameDifficulty( const int difficulty )
@@ -269,6 +288,7 @@ public:
     void setAutoSaveAtBeginningOfTurn( const bool enable );
     void setBattleDamageInfo( const bool enable );
     void setHideInterface( const bool enable );
+    void setNumericArmyEstimationView( const bool enable );
     void setScreenScalingTypeNearest( const bool enable );
 
     void SetSoundVolume( int v );
@@ -326,13 +346,13 @@ public:
         return players;
     }
 
-    int CurrentColor() const
+    PlayerColor CurrentColor() const
     {
         return players.getCurrentColor();
     }
 
     // The color should belong to one player or be NONE (neutral player).
-    void SetCurrentColor( const int color )
+    void SetCurrentColor( const PlayerColor color )
     {
         players.setCurrentColor( color );
     }
@@ -352,6 +372,16 @@ public:
         _viewWorldZoomLevel = zoomLevel;
     }
 
+    SaveFileSortingMethod getSaveFileSortingMethod() const
+    {
+        return _saveFileSortType;
+    }
+
+    void setSaveFileSortingMethod( const SaveFileSortingMethod sortType )
+    {
+        _saveFileSortType = sortType;
+    }
+
     void SetProgramPath( const char * path );
 
     static std::string GetVersion();
@@ -377,6 +407,8 @@ private:
     BitModes _editorOptions;
 
     fheroes2::ResolutionInfo _resolutionInfo;
+    fheroes2::Point _windowPos;
+
     int _gameDifficulty;
 
     std::string _programPath;
@@ -387,6 +419,8 @@ private:
 
     Maps::FileInfo _currentMapInfo;
 
+    SaveFileSortingMethod _saveFileSortType;
+
     int sound_volume;
     int music_volume;
     MusicSource _musicType;
